@@ -1,4 +1,4 @@
-# Copyright 2015-2017 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,14 +29,14 @@ class LineJoinerTest(yapf_test_helper.YAPFTest):
     style.SetGlobalStyle(style.CreatePEP8Style())
 
   def _CheckLineJoining(self, code, join_lines):
-    """Check that the given UnwrappedLines are joined as expected.
+    """Check that the given LogicalLines are joined as expected.
 
     Arguments:
       code: The code to check to see if we can join it.
       join_lines: True if we expect the lines to be joined.
     """
-    uwlines = yapf_test_helper.ParseAndUnwrap(code)
-    self.assertCodeEqual(line_joiner.CanMergeMultipleLines(uwlines), join_lines)
+    llines = yapf_test_helper.ParseAndUnwrap(code)
+    self.assertCodeEqual(line_joiner.CanMergeMultipleLines(llines), join_lines)
 
   def testSimpleSingleLineStatement(self):
     code = textwrap.dedent(u"""\
@@ -74,7 +74,7 @@ class LineJoinerTest(yapf_test_helper.YAPFTest):
   def testOverColumnLimit(self):
     code = textwrap.dedent(u"""\
         if instance(bbbbbbbbbbbbbbbbbbbbbbbbb, int): cccccccccccccccccccccccccc = ddddddddddddddddddddd
-        """)
+        """)  # noqa
     self._CheckLineJoining(code, join_lines=False)
 
 
