@@ -14,6 +14,7 @@
 # limitations under the License.
 """Tests for yapf.style."""
 
+import os
 import shutil
 import tempfile
 import textwrap
@@ -22,9 +23,10 @@ import unittest
 from yapf.yapflib import style
 
 from yapftests import utils
+from yapftests import yapf_test_helper
 
 
-class UtilsTest(unittest.TestCase):
+class UtilsTest(yapf_test_helper.YAPFTest):
 
   def testContinuationAlignStyleStringConverter(self):
     for cont_align_space in ('', 'space', '"space"', '\'space\''):
@@ -90,7 +92,7 @@ def _LooksLikeYapfStyle(cfg):
   return cfg['SPLIT_BEFORE_DOT']
 
 
-class PredefinedStylesByNameTest(unittest.TestCase):
+class PredefinedStylesByNameTest(yapf_test_helper.YAPFTest):
 
   @classmethod
   def setUpClass(cls):  # pylint: disable=g-missing-super-call
@@ -122,7 +124,7 @@ class PredefinedStylesByNameTest(unittest.TestCase):
       self.assertTrue(_LooksLikeFacebookStyle(cfg))
 
 
-class StyleFromFileTest(unittest.TestCase):
+class StyleFromFileTest(yapf_test_helper.YAPFTest):
 
   @classmethod
   def setUpClass(cls):  # pylint: disable=g-missing-super-call
@@ -201,8 +203,8 @@ class StyleFromFileTest(unittest.TestCase):
       self.assertEqual(cfg['I18N_FUNCTION_CALL'], ['N_', 'V_', 'T_'])
 
   def testErrorNoStyleFile(self):
-    with self.assertRaisesRegexp(style.StyleConfigError,
-                                 'is not a valid style or file path'):
+    with self.assertRaisesRegex(style.StyleConfigError,
+                                'is not a valid style or file path'):
       style.CreateStyleFromConfig('/8822/xyznosuchfile')
 
   def testErrorNoStyleSection(self):
@@ -211,8 +213,8 @@ class StyleFromFileTest(unittest.TestCase):
         indent_width=2
         ''')
     with utils.TempFileContents(self.test_tmpdir, cfg) as filepath:
-      with self.assertRaisesRegexp(style.StyleConfigError,
-                                   'Unable to find section'):
+      with self.assertRaisesRegex(style.StyleConfigError,
+                                  'Unable to find section'):
         style.CreateStyleFromConfig(filepath)
 
   def testErrorUnknownStyleOption(self):
@@ -222,12 +224,42 @@ class StyleFromFileTest(unittest.TestCase):
         hummus=2
         ''')
     with utils.TempFileContents(self.test_tmpdir, cfg) as filepath:
-      with self.assertRaisesRegexp(style.StyleConfigError,
-                                   'Unknown style option'):
+      with self.assertRaisesRegex(style.StyleConfigError,
+                                  'Unknown style option'):
         style.CreateStyleFromConfig(filepath)
 
+  def testPyprojectTomlNoYapfSection(self):
+    try:
+      import toml
+    except ImportError:
+      return
 
-class StyleFromDict(unittest.TestCase):
+    filepath = os.path.join(self.test_tmpdir, 'pyproject.toml')
+    _ = open(filepath, 'w')
+    with self.assertRaisesRegex(style.StyleConfigError,
+                                'Unable to find section'):
+      style.CreateStyleFromConfig(filepath)
+
+  def testPyprojectTomlParseYapfSection(self):
+    try:
+      import toml
+    except ImportError:
+      return
+
+    cfg = textwrap.dedent(u'''\
+        [tool.yapf]
+        based_on_style = "pep8"
+        continuation_indent_width = 40
+        ''')
+    filepath = os.path.join(self.test_tmpdir, 'pyproject.toml')
+    with open(filepath, 'w') as f:
+      f.write(cfg)
+    cfg = style.CreateStyleFromConfig(filepath)
+    self.assertTrue(_LooksLikePEP8Style(cfg))
+    self.assertEqual(cfg['CONTINUATION_INDENT_WIDTH'], 40)
+
+
+class StyleFromDict(yapf_test_helper.YAPFTest):
 
   @classmethod
   def setUpClass(cls):  # pylint: disable=g-missing-super-call
@@ -244,15 +276,15 @@ class StyleFromDict(unittest.TestCase):
     self.assertEqual(cfg['INDENT_WIDTH'], 2)
 
   def testDefaultBasedOnStyleBadDict(self):
-    self.assertRaisesRegexp(style.StyleConfigError, 'Unknown style option',
-                            style.CreateStyleFromConfig,
-                            {'based_on_styl': 'pep8'})
-    self.assertRaisesRegexp(style.StyleConfigError, 'not a valid',
-                            style.CreateStyleFromConfig,
-                            {'INDENT_WIDTH': 'FOUR'})
+    self.assertRaisesRegex(style.StyleConfigError, 'Unknown style option',
+                           style.CreateStyleFromConfig,
+                           {'based_on_styl': 'pep8'})
+    self.assertRaisesRegex(style.StyleConfigError, 'not a valid',
+                           style.CreateStyleFromConfig,
+                           {'INDENT_WIDTH': 'FOUR'})
 
 
-class StyleFromCommandLine(unittest.TestCase):
+class StyleFromCommandLine(yapf_test_helper.YAPFTest):
 
   @classmethod
   def setUpClass(cls):  # pylint: disable=g-missing-super-call
@@ -283,17 +315,15 @@ class StyleFromCommandLine(unittest.TestCase):
     self.assertIsInstance(cfg, dict)
 
   def testDefaultBasedOnStyleBadString(self):
-    self.assertRaisesRegexp(style.StyleConfigError, 'Unknown style option',
-                            style.CreateStyleFromConfig,
-                            '{based_on_styl: pep8}')
-    self.assertRaisesRegexp(style.StyleConfigError, 'not a valid',
-                            style.CreateStyleFromConfig, '{INDENT_WIDTH: FOUR}')
-    self.assertRaisesRegexp(style.StyleConfigError, 'Invalid style dict',
-                            style.CreateStyleFromConfig,
-                            '{based_on_style: pep8')
+    self.assertRaisesRegex(style.StyleConfigError, 'Unknown style option',
+                           style.CreateStyleFromConfig, '{based_on_styl: pep8}')
+    self.assertRaisesRegex(style.StyleConfigError, 'not a valid',
+                           style.CreateStyleFromConfig, '{INDENT_WIDTH: FOUR}')
+    self.assertRaisesRegex(style.StyleConfigError, 'Invalid style dict',
+                           style.CreateStyleFromConfig, '{based_on_style: pep8')
 
 
-class StyleHelp(unittest.TestCase):
+class StyleHelp(yapf_test_helper.YAPFTest):
 
   def testHelpKeys(self):
     settings = sorted(style.Help())
